# YASARA MOVIE
# TOPIC:    3. Molecular Dynamics
# TITLE:    3.7. Virtual reality simulations (PC)
# REQUIRES: Dynamics 23.4.1
# AUTHOR:   Kornel Ozvoldik, Elmar Krieger, Elina Krieger
# LICENSE:  GPL
# DESCRIPTION: A tutorial explaining how to work with MD simulations in VR

# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
OnError Continue
BallStickRadius 50,50
StickRadius 50
AtomSize 100
LightSource Alpha=020,Gamma=066,Ambience=085,Shadow=065
# Fresh start
Clear
# Left handed coordinate system
CoordSys Left
Projection perspective
UserInput Off
# Don't show console and head-up display
HUD Off
Console Off
MarkAtom None
# Can't use GPU for MD in VR
Processors GPU=0

Menu:
Clear
# List of sections
sectionlist=
  "Section 1: Hot air",
  "Section 2: Channel passage",
  "Section 3: Ligand tuning"

# Load background
bgr=LoadJPG sunset
ShowImage (bgr)
vron=Stereo
if !vron
  # Load restriction enzyme PVuII
  pvuobj = LoadPDB pvu2
  Style Ball
  HideAtom SideChain NucAcid
  for col in 'H,320','C,330','N,Blue','O,340','S,330','P,340'
    ColorAtom Obj (pvuobj) Element (col)
  # Create YASARA logo frame around it
  # The logo frame created by ShowLogoObj 189 A x 68.4 A tall and 9 A thick.
  lenx=189.
  leny=68.4e0
  lenz=9.
  # Build the logo frame from individual Xenon atoms to enable collisions during MD
  ShowLogoObj (pvuobj),0
  # Create the 'YASARA' letters
  textlist="YAS  RA","V"
  for i=1 to 2
    txtobj(i) = MakeTextObj YASARA,Width=200,Height=25
    PosText 50%,0%,justify=center
    Font Baikal,Height=100%,Color=White,Depth=2,DepthCol=404040
    Print (textlist(i))
  PrintCon
  MoveObj (txtobj2),X=8
  MoveObj (join txtobj),Y=-5,Z=-45
  Move Y=50,Z=250
  AutoRotateObj (txtobj2),X=1
  #779686,b9e3cf,b9e3cf,ffff90,
  ShowTiles 42,2,15,15,1.5,TileBottomLeft=ebfffe,97bda8,bae5d1,ffffff,Alpha=50%,SameCol=Yes,Joint=1.5,JointBottomLeft=83f7c2,ffffff,b9e3cf,779686,JointSide=2e7e4d
  RotateObj Tiles,90
  MoveObj Tiles,312,-45,100
  # Show small cell
  Cell 5,5,5
  cellpos()=-15,-35,120
  PosObj SimCell,(cellpos)
  # Load Yami and HMD
  LoadYami 0
  vpos()=PosObj (txtobj2)
  PosOriObj HMD,(vpos),0,0,0
  MoveObj HMD,0,100
  MoveObj Yami,100
  # Let logo fly in
  for i=1 to 3
    objpos()=PosObj (i)
    MoveObj (i),0,0,-300
    AutoPosObj (i),(objpos),240,0
  # Yami jumps into the scene
  for i=0 to 240
    yax=-0.5*i+120
    step=i%60
    AnimateYami yax,90,1.,step
    Wait 1
  for i=0 to 60
    AnimateYami 0,90-i,1.,i
    Wait 1
  Wait 10
  # Yami puts on HMD
  AutoPosOriObj HMD,X=3.000, Y=-23.000, Z=153.000, Alpha=-30.000, Beta=30.000, Gamma=0.000,Steps=20,Wait=Yes
  TransferObj HMD,Yami,Fix
  steps=80
  # Yami shrinks into the the cell
  AutoPosObj Yami HMD,(cellpos),(steps),Wait=No
  AutoRotateObj Yami HMD,1,2,3
  for i=1 to steps
    ScaleObj Yami HMD,.96,.96,.96
    if i==50
      StickObj Yami
    if i==80
      HideSecstrObj Yami
    Wait 1
# Reset scene
HUD Off
MarkAtom None
vron=Stereo
if vron
  DelAll
  LoadYami 1
  AutoPosOriObj Yami HMD,40,0,70,0,30,0,50,1
UserInput Off
colorlist='ff0000','ff8000','ffff00'
sections=count sectionlist
sectionsize=10
textobj = MakeTextObj Name=Sections,Width=60,Height=((sections-1)*sectionsize)
z=100
# Build menu
for i=1 to sections
  y=((0.5*(sections+1)-i)*sectionsize+1)
  color=colorlist(sections+1-i)
  # Create the atom to mark
  obj1 = BuildAtom Carbon
  ColorObj (obj1),(color)
  BallObj (obj1)
  # Create the donut around it
  obj2 = ShowTorus LargeRadius=2,LargeEdges=32,SmallRadius=0.5,SmallEdges=10,Color=(color)
  NameObj (obj2),'Option(i)'
  PosObj (join obj),-28,(y),(z)
  # Create the text
  Font Arial,Height=3,Color=(color),Depth=1.5,DepthCol=800000
  PosText 0%,(100.*(sections-i)/(sections-1))%,Left
  Print (sectionlist(i))
PrintCon
PosObj (textobj),8,0,(z)
obj=ListObj Methane
JoinObj Methane,(obj)
NumberObj all,1
# Load the menu
MoveObj Sections Methane Option?,X=-100,-10,15
AutoMoveObj Sections Methane Option?,X=2
Wait 50
AutoMoveObj Sections Methane Option?,X=0
# Wait until user selects an option
ShowMessage "Please click one circle to get specific usage instructions..."
# Take snapshot here, Eliza!
waits=0
do
  Wait 2
  opt = MarkAtom
  waits=waits+1
  if (waits>100)
    HideMessage
while opt<Atoms-sections+1
HideMessage
# The last atoms in the soup are our options, so opt is now 1..3
opt=opt-Atoms+sections
# Let options fly away
AutoRotateObj Option(opt),X=1,Y=3,Z=2
Wait 60
AutoRotateObj Sections,Y=1
AutoMoveObj Sections,Z=-2
AutoMoveObj Option? Methane,Y=-1
Wait 100
if vron
  AutoPosOriObj Yami HMD,0,0,50,0,0,0,50,1
  if opt==2
    # Let Yami jump to the VR area
    _,_,_,areapos()=VRArea
    yamipos()=areapos1,areapos2+22,areapos3
    jumppos()=yamipos/2
    jumppos2=100
    AutoPosObj Yami HMD,(jumppos),Steps=100,Wait=Yes
    AutoPosObj Yami HMD,(yamipos),Steps=100,Wait=Yes
  elif opt==3
    # Let Yami vanish
    DelObj not Yami HMD
    cut=Cut
    RotateObj (cut),90
    MoveObj (cut),Z=-100
    AutoRotateObj (cut),0,1
    AutoMoveObj (cut),Z=.2
    Wait 360
    AutoRotateObj (cut),0,0
    AutoMoveObj (cut),Z=0
    Wait 10
    DelObj Yami (cut)
    CenterObj HMD
    _,_,_,_,floory=VRArea
    AutoRotate 0,0,10
    Wait 10
    AutoRotate 0,0,-10
    Wait 20
    AutoRotate 0,0,10
    Wait 20
    AutoRotate 0,0,-10
    Wait 20
    AutoRotate 0,0,10
    Wait 10
    AutoRotate 0,0,0
    Wait 20
    AutoPosObj HMD,Y=(floory),Steps=20
    Del
else
  ShowMessage "Follow Yami into VR! Click Window > Stereo > Virtual reality"
  do
    Wait 2
    vron=Stereo
  while !vron
# Go to selected section
Go Section(opt)

#
# Section 1 - MD simulation of hot air
#
Section1:
StartSection
LoadYami 1
FixObj Yami
ColorObj Yami,Element
ShowMessage "Click the 'Continue' button to create a box with air..."
Wait ContinueButton
_,_,_,areapos()=VRArea
cellsize=200
Cell (cellsize),(cellsize),(cellsize)
PosObj SimCell,0,(cellsize/2+areapos2),0
# Fill it with air
oxyobj = BuildSMILES "O=O"
nitobj = BuildSMILES "N#N"
co2obj = BuildSMILES "O=C=O"
NameObj (oxyobj),Oxygen
NameObj (nitobj),Nitrogen
NameObj (co2obj),CO2
density=16
PosObj (oxyobj),0,0,10
FillCellObj (nitobj),Copies=(77*density)
FillCellObj (co2obj),Copies=( 3*density)
BallObj (oxyobj) (nitobj) (co2obj)
# Force field parameters
ForceField AMBER14
Cutoff 5
Longrange None
Boundary Wall
TimeStep 3,2.5
Temp 2000K
Brake 6000
textlist="You are inside a box with air that has a bug: mostly nitrogens, but only 1 oxygen.",
         'Click Simulation > Fill cell with > Copies, select Oxygen and (20*density) copies.',
         "Now let's start the simulation: Click Simulation > Simulator > On or press F12",
         "Choose a slow molecule and try to mark it with the controller beam.",
         '(buttondownhold) to pull the marked atom to the controller.',
         '(buttonuphold) to push the atom away along the beam.',
         "Additionally pull the trigger fully to push or pull the whole object.",
         "Let's play a game: try to click as many atoms as possible within 15 picosecs."
UserInput On
for i=1 to count textlist
  ShowMessage (textlist(i))
  Wait ContinueButton
  if i==2
    oxys=CountAtom Object (oxyobj)
    if oxys==2
      FillCellObj Oxygen,Copies=(20*density),BumpSum=1.0,RandomOri=0,DisMin=0.0
    TempCtrl SteepDes
    Sim On
    Wait SpeedMax<3000
    Wait 20
    Sim Off
    TempCtrl Rescale
  if i==3
    Sim On
    PullForce 16
# The video game for fast clickers
LabelPar Font=Arial
while 1
  Color Element
  Sim On
  ShowMessage "The faster the molecule you click, the higher the score. Click 'Continue' to start."
  Wait ContinueButton
  HideMessage
  Time 0
  score=0.
  lastt=0
  MarkAtom None
  # Main game loop
  do
    Wait 5
    atm = MarkAtom
    marked = CountAtom (atm) and not Obj Yami
    if marked
      speed = SpeedRes Atom (atm)
      score=score+speed/1000
      ColorRes Atom (atm),Yellow
      Wait 5
      HideRes Atom (atm)
      MarkAtom None
    t = Time
    t=15-t/1000
    if t!=lastt or score!=lastscore
      # Update Score and Time label only when needed
      lastt=t
      lastscore=score
      UnlabelAll
      LabelAll 'Score (0.0+score)',Height=0.5,Color=Yellow,X=-2.5,Y=1.5
      LabelAll 'Time (t)',Height=0.5,Color=Yellow,X=2.5,Y=1.5
  while t>0
  LabelAll 'Your score is (0.0+score)',Height=1,Color=Red,Y=0
  ShowMessage 'Molecules you hit are colored yellow.'
  ShowRes UNL CO2
  Wait ContinueButton
  MarkAtom None
  ShowMessage '(buttonrightclick) to play again or continue...'
  ShowButton Continue
  while 1
    if ContinueButton
      # Break out of game loop
      break 2
    if RightButton
      # Continue game loop
      continue 2
    Wait 1

#
# Section 2 - Ion channel passage
#
# https://en.wikipedia.org/wiki/Two-pore-domain_potassium_channel
# PDB entry 7SK0 paper: https://www.nature.com/articles/s41467-022-30853-z
# Channel gating schematics: https://molpharm.aspetjournals.org/content/90/3/309
# Potassium flow direction: https://www.sciencedirect.com/science/article/pii/S0022283621001960 and https://www.guidetopharmacology.org/GRAC/FamilyIntroductionForward?familyId=79
# No water in potassium channel: https://www.sciencedaily.com/releases/2019/08/190809104945.htm
# Barium blocks potassium channels: //www.sciencedirect.com/science/article/abs/pii/S0022283620304083
Section2:
StartSection
ShowMessage "Click the 'Continue' button to load an ion channel..."
Wait ContinueButton
LoadSce 7sk0
LightSource Ambience=85
Style Ribbon,Sidechain=Stick
StickRadius 30
BallObj Membrane Water
StickRes Water
_,_,beta,areapos()=VRArea
PosOri (areapos),0,(beta)
_,sizey,sizez=Cell
Move 0,(sizey/2),(-sizez/2)
# Bad barium blocking the selectivity filter
barium=ListAtom Element Barium
# Lonely potassium that wants to return to its friends
potassium=ListAtom Element Potassium and LocalY>0
# Color barium orange
ColorAtom (barium),154
ColorAtom Element Potassium,Magenta
# Selectivity filter oxygens
filter1()=ListAtom O Res TYR 120 LEU 228
filter2()=ListAtom O Res GLY 119 GLY 227
filter3()=ListAtom O Res THR 118 ILE 226
filter4()=ListAtom O Res THR 117 THR 225
filter5()=ListAtom OG1 Res THR 117 THR 225
# Show selectivity filter
ShowRes Atom (join filter)
BallStickRes Atom (join filter)
StickAtom SideChain
BallStickRes Atom (join filter5)
BallAtom (barium)
# Set simulation parameters
ForceField AMBER14,SetPar=Yes
TimeStep 2,2
Temp 25C
Brake 6000
for i=1 to 5
  # Keep Selectivity filter intact during simulation
  AddSpring (join filter(i)),(join filter(i))
UserInput On
HUD Obj
textlist="Stand up, take the GUI with you and walk around the VR area...",
         "...examining the two pore domain potassium channel TWIK-1 (PDB ID 7SK0)!",
         "Let's see it in action! Click on the right to activate the VR keyboard and press F12."
ShowMessages textlist
Sim On
HUD Sim
PullForce 32
textlist="On the extracellular side on the top, the sodium (yellow balls) concentration is high.",
         "Go on your knees and verify that potassium (magenta) dominates intracellularly.",
         "But wait - a single magenta potassium ion got outside. Can you find and mark it?"
ShowMessages textlist
MarkAtom (potassium)
textlist="It feels lonely and wants to return back into the cell to its potassium friends!",
         "Due to the concentration gradient, this will require energy and thus your help pulling."
ShowMessages textlist
ShowMessage "But wait again - an evil barium ion is blocking the channel's selectivity filter. Mark it!"
do
  atm=MarkAtom
  Wait 10
while atm!=barium
textlist="The selectivity filter consists of 4 rings of 4 carbonyls, delineating sites S1-4...",
         "...and a fifth ring of four hydroxyls at the ion dehydration site S0 at the bottom.",
         "Barium is known as a permanent blocker of potassium channels. Time to pull it out..."
ShowMessages textlist
MarkAtom (barium)
do
  atm=MarkAtom
  if atm!=barium
    MarkAtom (barium)
  ShowMessage '(buttondownhold) to pull the barium ion out at the bottom.'
  Wait 10
  dis=Distance (barium),(filter31)
while dis<20
MarkAtom (potassium)
textlist="Now that the filter is free, let's help our lonely potassium.",
         "While pulling is easier, you can also try to push it in the direction of the laser:",
         '(buttonuphold) to push a marked atom along the beam.'
ShowMessages textlist
MarkAtom (potassium)
for i=1 to 5
  SelectAtom (join filter(i))
  do
    _,kposy=PosAtom (potassium)
    _,oposy=PosAtom (filter(i)1)
    ShowMessage 'Push or pull the potassium ion to site S(5-i)'
    Wait 2
  while kposy>oposy+1
  Unselect
ShowMessage "Well done! Sit down again and press 'Continue' to proceed to the next section..."
Wait ContinueButton
StickRadius 50

#
# Section 3 - Improve ligand
#
Section3:
StartSection
# Includes "RemoveEnvRes all with distance>10 from DJR" to boost performance on Android VR
LoadSce building1
UserInput on
Pos Z=20
LabelRes Pro 81 Mol B,Format="RESName RESNUM",Height=0.5
LabelRes Ile 50 Mol A,Format="RESName RESNUM",Height=0.5,X=-0.5,Y=-1,Z=-1
ShowMessage "Let's continue with a drug design example."
Wait ContinueButton
ShowMessage "To improve the ligand, we will increase its hydrophobic interactions..."
Wait ContinueButton
ShowMessage "...by squeezing a phenyl ring between Ile 50 and Pro 81."
for i=85 to 40 step -1
  ColorSurfObj Receptor,OutAlpha=(i),InAlpha=60
  Wait 1
ShowArrow Start=AtAtom,2840,End=AtAtom,798,Radius=0.5,Heads=1,Color=Magenta
LabelDis 2840,798,Format="DIS A",Height=0.7,Z=-0.5
Wait ContinueButton
ForceField NOVA,SetPar=Yes
Boundary Periodic
SimSteps 1
textcom="Click 'View > Show atoms > Residue', double-click 'Djr' to bring hydrogens back.",-1,'ShowRes DJR',
        "This approach has been described by Arun K. Ghosh et al., let's try it...",-1,'pass',
        "Click the benzene-icon at the top to activate the molecular builder...",-1,'ColorAtom H13 Res DJR,Yellow',
        "Click on the yellow hydrogen, then on the Phenyl group in the builder.",3206,'SwapAtom 3161,Phenyl,AttachPoint=1',
        "The new ring strongly bumps into Ile 50, so the rings cannot be co-planar.",-1,'pass',
        "To change a dihedral, mark it: click atom 1, then...",-5,'MarkAtom 3172',
        '...hold the Ctrl key and click the other three.',-5,'MarkAtom 3172,3160,3161,3162',
        "Right-click on the marked atom, select 'Geometry > Dihedral', move it to -50.",-1,'Dihedral 3172,3160,3161,3162,set=-50',
        "Finally, Ghosh et al. added an orto-methoxy group...",-1,'ColorAtom 3163,Yellow',
        "Click on the yellow hydrogen,then click the Methoxy-group in the builder.",3216,'SwapAtom 3163,Methoxy',
        "To get rid of the clashes, an energy minimization is needed.",-6,'pass',
        "Click 'Options > Choose experiment > Energy minimization'.",-4,'LoadSce building2',
        "Note how the distance between Ile 50 and Pro 81 increased to 7 A...",-1,'pass',
        "...and the surface now shows a cleft that accomodated the phenyl.",-1,'HideAtom Element H with bond to Element C'
ShowTutorial textcom

#
# The End
#
TheEnd:
StartSection
Del
UserInput off
# Load and replicate the Klingon warbirds
LoadYOb pyromellitimide
Style Ball
molecules=80
for i=2 to molecules
  DuplicateObj 1
for i=1 to molecules
  PosObj (i),(rnd 200-100),(rnd 200-100),(rnd 1500+1300)
ShowMessage "Hopefully, you enjoyed this VR tutorial!"
Wait ContinueButton
ShowMessage "Click 'Continue', then watch out for a surprise. Ready your beams!"
Wait ContinueButton
ShowMessage "Click 'Continue' to end."
ShowButton Continue
for i=1 to molecules
  RotateObj (i),X=-90
  AutoRotateObj (i),Z=(rnd 5-2.5)
  AutoMoveObj (i),Z=-(rnd 1.5+1.5)
# Keep Klingon warbirds on screen
while 1
  for i=1 to molecules
    x,y,z=PosObj (i)
    if z<-1300
      MoveObj (i),Z=2600
    if ContinueButton
      break 2
    Wait 1
    atm = MarkAtom
    marked = CountAtom (atm)
    if marked
      ColorRes Atom (atm),Yellow
      Wait 5
      HideRes Atom (atm)
      MarkAtom None

# SHOW A LIST OF TEXT MESSAGES
# ============================
def ShowMessages textlist()
  for i=1 to count textlist
    ShowMessage (textlist(i))
    Wait ContinueButton

# MOVE YAMI ALONG
# ===============
# x IS THE X-POSITION, beta THE BETA ORIENTATION ANGLE, tilt THE AMOUNT OF TILTING (~SPEED),
# step IS THE ANIMATION STEP
def AnimateYami x,beta,tilt,step
  step=step%60
  # Place Yami
  height=sin (180.*step/60)*18
  PosObj Yami HMD,(x),(height-20),150
  OriObj Yami HMD,0,0,0
  RotateObj Yami HMD,X=(-sin (360.*step/60)*30*tilt),Y=(beta)

# LOAD YAMI WITH HMD ON/OFF
# =========================
def LoadYami hmdon
  yami=LoadYOb yami
  NameObj (yami),Yami
  tex=LoadJPG generic_hmd
  hmd=LoadWOb generic_hmd,Open=No,(tex)
  NameObj (hmd),HMD
  scale=75
  ScaleObj HMD,(scale),(scale),(scale)
  PosOriObj HMD, X=-1.601, Y=4.441, Z=41.296, Alpha=8.418, Beta=-168.021, Gamma=-3.742
  if !hmdon
    MoveObj HMD,Y=7
    RotateObj HMD,30
  TransferObj HMD,Yami,Fix

# PREPARE SCENE FOR NEW SECTION
# =============================
def StartSection
  global buttonmenu,buttonleftclick,buttonlefthold,buttonrightclick,buttonrighthold,buttonupclick,buttonuphold,buttondownclick,buttondownhold,buttoncenterclick
  # Reset scene
  Stereo VR
  DelAll
  DelImage All
  # Assign button descriptions depending on headset
  buttonupclick='Tap joystick up'
  buttonuphold='Hold joystick up'
  buttondownclick='Tap joystick down'
  buttondownhold='Hold joystick down'
  buttoncenterclick='Click the joystick'
  if Headset=='Vive' or Headset=='Vive Pro' or Headset=='Vive Pro 2' or Headset=='Vive Cosmos Elite'
    # Vive
    buttonmenu='Menu button'
    buttonleftclick='Click trackpad left'
    buttonlefthold='Hold trackpad left'
    buttonrightclick='Click trackpad right'
    buttonrighthold='Hold trackpad right'
    buttonupclick='Click trackpad up'
    buttonuphold='Hold trackpad up'
    buttondownclick='Click trackpad down'
    buttondownhold='Hold trackpad down'
    buttoncenterclick='Click trackpad in the center'
  elif Headset=='Valve Index'
    # Valve Index
    buttonmenu='B button'
    buttonleftclick='Click left A or tap joystick left'
    buttonlefthold='Hold left A or hold joystick left'
    buttonrightclick='Click right A or tap joystick right'
    buttonrighthold='Hold right A or hold joystick right'
  else
    # Meta Quest 2 and any other headset
    buttonmenu='Y or B button'
    buttonleftclick='Click X or tap joystick left'
    buttonlefthold='Hold X or hold joystick left'
    buttonrightclick='Click A or tap joystick right'
    buttonrighthold='Hold A or hold joystick right'

# SHOW A TUTORIAL
# ===============
def ShowTutorial textcom()
  # Delete leftovers
  UserInput On
  # Hidden command to keep undo/redo active during movies, helps in case of building accidents
  KeepTab
  # Show the messages
  for i=1 to count textcom step 3
    ShowMessage (textcom(i))
    # textcom(i+1) IS -1 TO EXECUTE ALWAYS, -2 TO EXECUTE ONLY IF NO CELL IS PRESENT, -3 NOT TO WAIT,
    if textcom(i+1)!=-3
      Wait ContinueButton
    cellobj = ListObj SimCell
    if textcom(i+1)==Atoms or textcom(i+1)==-1 or (textcom(i+1)==-2 and !cellobj) or textcom(i+1)<-2 and textcom(i+1)!=-4
      # Number of atoms unchanged, assume that user just clicked 'Continue', or always execute
      (textcom(i+2))
    if textcom(i+1)==-4
      # Create second view if missing
      views=DuplicateView Main,tmp
      DelView tmp
      if views==2
        (textcom(i+2))
      ShowView 2
    if textcom(i+1)==-5
      # Switch to faster molecular surface for interactive dihedral changes+minimization
      SurfPar Molecular=Gaussian
    if textcom(i+1)==-6
      # Prepare minimization
      TempCtrl SteepDes
      Sim On
      Wait SpeedMax<3000
      Sim Off  
